#!/bin/bash

usage() {
    cat <<-EOF
	usage: $SCRIPT_NAME [OPTIONS]...
	Copy image to nandflash

	OPTIONS:
	  -b BOARD    board ${BOARD/*/(default=$BOARD)}
	  -d DEV      device ${DEV/*/(default=$DEV)}
	  -s SCRIPT   load script ${SCRIPT/*/(default=$SCRIPT)}
	  -h          help
EOF
}

#set -e

SCRIPT_HOME=$(cd $(dirname ${BASH_SOURCE[0]}) && pwd)
SCRIPT_NAME=$(basename ${BASH_SOURCE[0]})

BOARD=at91sam9rl64-ek
DEV=/dev/ttyACM0
SCRIPT=sam-ba-nandflash.tcl

# Parse command line
while [ $# -gt 0 ]; do
    case "$1" in
	-d|--dev)    shift; DEV=$1;;
	-b|--board)  shift; BOARD=$1;;
	-s|--script) shift; SCRIPT=$1;;
	-h|--help)   usage; exit 0;;
	*)           usage; exit 1;;
    esac
    shift
done

echo "Copy image to nand flash on AT91SAM9RL-EK with sam-ba"
echo "1. Connect USB cable between PC and target (J21)"
echo "2. Power off target"
echo "3. Open jumpers J12 + J13"
echo "4. Power on target"
echo "5. Close jumpers J12 + J13"

read -p "Continue [y/N] : " ANSWER
case "$ANSWER" in
    y|Y|yes|YES);;
    *) exit 0;;
esac

echo "sam-ba $DEV $BOARD $SCRIPT..." 
sam-ba $DEV $BOARD $SCRIPT

echo "done"

echo "Push RESET button to reboot"
